"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const actions_1 = require("./actions");
const tags_1 = require("./tags");
const getClient = (clientId, state) => state.items[state.mappings[clientId]];
const setClient = (clientId, client, state) => {
    const items = state.items.slice();
    items[state.mappings[clientId]] = client;
    return {
        items,
        mappings: state.mappings
    };
};
exports.combineClients = (clientReducer) => (state, action) => {
    if (typeof state === 'undefined') {
        return {
            items: [],
            mappings: {}
        };
    }
    if (action.type === actions_1.PRESENT) {
        const { clients, shared } = action.payload.state;
        const items = clients.items.map((client) => clientReducer(client, { type: actions_1.PRESENT, payload: { state: { shared, client } } }));
        return { items, mappings: clients.mappings };
    }
    if (action.type === actions_1.ADD_CLIENT) {
        if (typeof state.mappings[action[tags_1.CLIENT]] === "undefined") {
            const items = [...state.items, clientReducer({ id: action[tags_1.CLIENT] }, action)];
            return {
                items,
                mappings: Object.assign({}, state.mappings, { [action[tags_1.CLIENT]]: items.length - 1 })
            };
        }
        else {
            return state;
        }
    }
    return (state && typeof action[tags_1.CLIENT] !== 'undefined')
        ? setClient(action[tags_1.CLIENT], clientReducer(getClient(action[tags_1.CLIENT], state), action), state)
        : {
            items: state.items.map(client => clientReducer(client, action)),
            mappings: state.mappings
        };
};
//# sourceMappingURL=combineClients.js.map