"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const actions_1 = require("./actions");
const tags_1 = require("./tags");
const debug = require('debug')('redux-socket-client');
function middleware(socket, options = { clientFirst: false }) {
    const setupSockets = [];
    const autoInit = arguments.length === 2 || (arguments.length === 1 && typeof socket.clientFirst === 'undefined');
    if (!autoInit) {
        options = socket;
        socket = null;
    }
    const middleware = (store) => {
        const SERVER = Symbol('server');
        const queue = [];
        let version = -100;
        let clientFirst = options && options.clientFirst;
        const processAction = (action) => {
            action[SERVER] = true;
            debug('incoming action', action);
            store.dispatch(action);
            version++;
        };
        const handlePresent = (present) => {
            version = -100;
            store.dispatch({
                [SERVER]: true,
                type: actions_1.PRESENT,
                payload: { state: present.state }
            });
            version = present.version;
            while (queue.length) {
                const { version: next, action } = queue.pop();
                if (clientFirst && next !== version + 1)
                    continue;
                processAction(action);
            }
        };
        const handleVersion = (v) => version = v;
        const handleAction = (data) => {
            if (clientFirst && data.version !== version + 1)
                return queue.unshift(data);
            if (data.client)
                data.action[tags_1.CLIENT] = data.client;
            processAction(data.action);
        };
        let emit = null;
        function setupSocket(newSocket) {
            if (!newSocket)
                return;
            emit = newSocket.emit.bind(newSocket);
            if (socket && newSocket !== socket) {
                socket.removeListener('present', handlePresent);
                socket.removeListener('version', handleVersion);
                socket.removeListener('action', handleAction);
            }
            newSocket.on('present', handlePresent);
            newSocket.on('version', handleVersion);
            newSocket.on('action', handleAction);
            emit('present');
        }
        setupSockets.push(setupSocket);
        if (autoInit)
            setupSocket(socket);
        return (next) => (action) => {
            if (action.type === actions_1.CONNECT) {
                setupSocket(action.payload.socket);
            }
            else if (emit) {
                if (action[SERVER]) {
                    delete action[SERVER];
                    return next(action);
                }
                if (action[tags_1.SHARED]) {
                    debug('outgoing action', action);
                    emit(action[tags_1.CLIENT] ? 'client-action' : 'action', action);
                    if (clientFirst)
                        return next(action);
                    else
                        return next({ type: actions_1.SKIP, payload: action });
                }
            }
            return next(action);
        };
    };
    middleware.switchSocket = (socket) => {
        for (let setupSocket of setupSockets)
            setupSocket(socket);
    };
    return middleware;
}
exports.sharedStoreMiddleware = middleware;
//# sourceMappingURL=middleware.js.map